#ifndef __pci_h__
#define __pci_h__

#include "type.h"
#include "io.h"

template<class type>
struct PciConfAccessor
{
	Relative<U32, 0,Io<U32>  > addr;
	Relative<type,4,Io<type> > data;
};


#define pciBsf(bus,slot,function) ((0x80<<24)|(bus<<16) | (slot<<11) | (function<<8))

template<class type>
struct PciConfReg
{
	type justForPointerArithmetic;

#if 0
	static PciConfAccessor<type> * const pciConfAccessor=((PciConfAccessor<type>* const)0xcf8);

	operator type()
	{
		pciConfAccessor->addr=(U32)this;
		return pciConfAccessor->data;
	}
	PciConfReg<type>& operator =(type v)
	{
		pciConfAccessor->addr=(U32)this;
		pciConfAccessor->data=v;
		return *this;
	}
#else
	operator type()
	{
		((PciConfAccessor<type>* const)0xcf8)->addr=(U32)this;
		return ((PciConfAccessor<type>* const)0xcf8)->data;
	}
	PciConfReg<type>& operator =(type v)
	{
		((PciConfAccessor<type>* const)0xcf8)->addr=(U32)this;
		((PciConfAccessor<type>* const)0xcf8)->data=v;
		return *this;
	}
#endif
};

struct PciConf
{
	inline static PciConf* bsf(U32 bus,U32 slot,U32 function)
	{
		return (PciConf*) (pciBsf(bus,slot,function));
	}
	RelativeRef<PciConfReg<U32> ,0x00> regs;
	Relative<U32,0x00,PciConfReg<U32> > vendorDeviceId;
	Relative<U16,0x00,PciConfReg<U16> > vendorId;
	Relative<U16,0x02,PciConfReg<U16> > deviceId;
	Relative<U32,0x04,PciConfReg<U32> > commandStatus;
	Relative<U16,0x04,PciConfReg<U16> > command;
	Relative<U16,0x06,PciConfReg<U16> > status;
	Relative<U8 ,0x08,PciConfReg<U8>  > revision;
	Relative<U8 ,0x09,PciConfReg<U8>  > programmingIf;
	Relative<U8 ,0x0a,PciConfReg<U8>  > subclass;
	Relative<U8 ,0x0b,PciConfReg<U8>  > mainclass;
	Relative<U8 ,0x0c,PciConfReg<U8>  > cacheLineSize;
	Relative<U8 ,0x0d,PciConfReg<U8>  > latencyTimer;
	Relative<U8 ,0x0e,PciConfReg<U8>  > headerType;
	Relative<U8 ,0x0f,PciConfReg<U8>  > bist;
	RelativeRef<PciConfReg<U32> ,0x10> baseAddresses;
	Relative<U32,0x28,PciConfReg<U32> > cis;
	Relative<U16,0x2c,PciConfReg<U16> > subVendorId;
	Relative<U16,0x2e,PciConfReg<U16> > systemId;
	Relative<U32,0x30,PciConfReg<U32> > expansionRom;
	Relative<U32,0x34,PciConfReg<U32> > reserved0;
	Relative<U32,0x38,PciConfReg<U32> > reserved1;
	Relative<U8 ,0x3c,PciConfReg<U8>  > intLine;
	Relative<U8 ,0x3d,PciConfReg<U8>  > intPin;
	Relative<U8 ,0x3e,PciConfReg<U8>  > minGrant;
	Relative<U8 ,0x3f,PciConfReg<U8>  > maxLatency;
	Relative<U8 ,0x41,PciConfReg<U8>  > specialUse;
	Relative<U8 ,0x43,PciConfReg<U8>  > mode;
};

typedef struct OnDevice
{
	U32 vendorDevice;
	void (*handle)(PciConf *pci);
}OnDevice;

extern void pciScan(const OnDevice *onDevices);

#endif /* __pci_h__ */
